from json import loads
from pathlib import Path


_cidades = None


def ler_cidades():
    global _cidades
    if _cidades is not None:
        return _cidades

    with open(Path(__file__).parent / "municipios.json") as f:
        _cidades = loads(f.read())
    return _cidades


def validar_codigo_cidade(codigo: int) -> bool:
    cidades = ler_cidades()
    return any(cidade["id"] == codigo for cidade in cidades)


def normalizar_uf(uf: str) -> str:
    cidades = ler_cidades()
    uf = uf.upper()
    assert any(
        cidade["regiao-imediata"]["regiao-intermediaria"]["UF"]["sigla"] == uf
        for cidade in cidades
    ), "UF não encontrada. Insira uma UF válida no formato de sigla (ex: SP, RJ, MG, etc)"
    return uf



from abc import ABC, abstractmethod
from abstra_notas.assinatura import Assinador
from .retorno import Retorno
from .templates import load_template
from jinja2 import Template


class Pedido(ABC):
    @property
    @abstractmethod
    def classe_retorno(self) -> Retorno:
        raise NotImplementedError

    @abstractmethod
    def gerar_xml(self, assinador: Assinador) -> str:
        raise NotImplementedError

    @property
    def template(self) -> Template:
        return load_template(self.__class__.__name__)



from dataclasses import dataclass
from lxml.etree import Element, SubElement


@dataclass
class Detalhe:
    inscricao_municipal: str
    emite_nfse: bool

    def gerar_xml(self):
        detalhe = Element("Detalhe")
        SubElement(detalhe, "InscricaoMunicipal").text = self.inscricao_municipal
        SubElement(detalhe, "EmiteNFSe").text = "true" if self.emite_nfse else "false"
        return detalhe

    @staticmethod
    def parse_xml(element: Element) -> "Detalhe":
        inscricao_municipal = element.find("InscricaoMunicipal").text
        emite_nfse = element.find("EmiteNFSe").text == "true"
        return Detalhe(inscricao_municipal=inscricao_municipal, emite_nfse=emite_nfse)


@dataclass
class RetornoConsultaCNPJ:
    sucesso: bool
    detalhe: Detalhe

    def gerar_xml(self):
        retorno_consulta_cnpj = Element(
            "p1:RetornoConsultaCNPJ",
            nsmap={
                "p1": "http://www.prefeitura.sp.gov.br/nfe",
                "xsi": "http://www.w3.org/2001/XMLSchema-instance",
            },
        )
        SubElement(retorno_consulta_cnpj, "Sucesso").text = (
            "true" if self.sucesso else "false"
        )
        detalhe = self.detalhe.gerar_xml()
        retorno_consulta_cnpj.append(detalhe)
        return retorno_consulta_cnpj

    @staticmethod
    def parse_xml(element: Element) -> "RetornoConsultaCNPJ":
        sucesso = element.find("Sucesso").text == "true"
        detalhe = Detalhe.parse_xml(element.find("Detalhe"))
        return RetornoConsultaCNPJ(sucesso=sucesso, detalhe=detalhe)



def normalizar_cpf(cpf: str) -> str:
    cpf = cpf.replace(".", "").replace("-", "")

    if len(cpf) == 11:
        validar = True
        digitos_verificadores = cpf[9:]
    else:
        validar = False

    cpf = cpf[:9]

    try:
        dig_1 = int(cpf[0]) * 1
        dig_2 = int(cpf[1]) * 2
        dig_3 = int(cpf[2]) * 3
        dig_4 = int(cpf[3]) * 4
        dig_5 = int(cpf[4]) * 5
        dig_6 = int(cpf[5]) * 6
        dig_7 = int(cpf[6]) * 7
        dig_8 = int(cpf[7]) * 8
        dig_9 = int(cpf[8]) * 9
    except IndexError:
        raise Exception("Quantidade de caracteres incorreto.")

    dig_1_ao_9_somados = (
        dig_1 + dig_2 + dig_3 + dig_4 + dig_5 + dig_6 + dig_7 + dig_8 + dig_9
    )

    dig_10 = dig_1_ao_9_somados % 11

    if dig_10 > 9:
        dig_10 = 0

    cpf += str(dig_10)

    dig_1 = int(cpf[0]) * 0
    dig_2 = int(cpf[1]) * 1
    dig_3 = int(cpf[2]) * 2
    dig_4 = int(cpf[3]) * 3
    dig_5 = int(cpf[4]) * 4
    dig_6 = int(cpf[5]) * 5
    dig_7 = int(cpf[6]) * 6
    dig_8 = int(cpf[7]) * 7
    dig_9 = int(cpf[8]) * 8
    dig_10 = int(cpf[9]) * 9

    dig_1_ao_10_somados = (
        dig_1 + dig_2 + dig_3 + dig_4 + dig_5 + dig_6 + dig_7 + dig_8 + dig_9 + dig_10
    )

    dig_11 = dig_1_ao_10_somados % 11

    if dig_11 > 9:
        dig_11 = 0

    cpf_validado = cpf + str(dig_11)

    cpf = (
        cpf_validado[:3]
        + "."
        + cpf_validado[3:6]
        + "."
        + cpf_validado[6:9]
        + "-"
        + cpf_validado[9:]
    )

    cpf_normalizado = cpf.replace(".", "").replace("-", "")
    if validar:
        if digitos_verificadores == cpf_validado[9:]:
            return cpf_normalizado
        else:
            print()
            raise Exception("Os dígitos verificadores estão incorretos.")
    else:
        return cpf_normalizado



from .pedido import Pedido
from dataclasses import dataclass
from lxml.etree import Element
from abstra_notas.assinatura import Assinador
from typing import Tuple, Literal


@dataclass
class RetornoConsultaCNPJ:
    sucesso: bool
    inscricao_municipal: str
    emite_nfe: bool

    @staticmethod
    def ler_xml(xml: Element) -> "RetornoConsultaCNPJ":
        return RetornoConsultaCNPJ(
            sucesso=xml.find(".//Sucesso").text == "true",
            inscricao_municipal=xml.find(".//InscricaoMunicipal").text,
            emite_nfe=xml.find(".//EmiteNFe").text == "true",
        )


@dataclass
class PedidoConsultaCNPJ(Pedido):
    remetente: Tuple[Literal["CNPJ", "CPF"], str]
    contribuinte: Tuple[Literal["CNPJ", "CPF"], str]

    @property
    def classe_retorno(self) -> RetornoConsultaCNPJ:
        return RetornoConsultaCNPJ

    def gerar_xml(self, assinador: Assinador):
        xml = self.template.render(
            remetente=self.remetente,
            contribuinte=self.contribuinte,
        )

        return xml



from pathlib import Path
from jinja2 import Template

current_path = Path(__file__).parent

def load_template(class_name):
    with open(current_path / f"{class_name}.xml", "r") as file:
        return Template(file.read())


def normalizar_cnpj(cnpj: str) -> str:
    cnpj = cnpj.replace(".", "").replace("/", "").replace("-", "")

    if len(cnpj) == 14:
        validar = True
        digitos_verificadores = cnpj[12:]
    else:
        validar = False

    cnpj = cnpj[:12]

    try:
        dig_1 = int(cnpj[0]) * 6
        dig_2 = int(cnpj[1]) * 7
        dig_3 = int(cnpj[2]) * 8
        dig_4 = int(cnpj[3]) * 9
        dig_5 = int(cnpj[4]) * 2
        dig_6 = int(cnpj[5]) * 3
        dig_7 = int(cnpj[6]) * 4
        dig_8 = int(cnpj[7]) * 5
        dig_9 = int(cnpj[8]) * 6
        dig_10 = int(cnpj[9]) * 7
        dig_11 = int(cnpj[10]) * 8
        dig_12 = int(cnpj[11]) * 9
    except IndexError:
        raise Exception("Quantidade de caracteres incorreto.")

    dig_1_ao_12_somados = (
        dig_1
        + dig_2
        + dig_3
        + dig_4
        + dig_5
        + dig_6
        + dig_7
        + dig_8
        + dig_9
        + dig_10
        + dig_11
        + dig_12
    )

    dig_13 = dig_1_ao_12_somados % 11

    if dig_13 > 9:
        dig_13 = 0

    cnpj += str(dig_13)

    dig_1 = int(cnpj[0]) * 5
    dig_2 = int(cnpj[1]) * 6
    dig_3 = int(cnpj[2]) * 7
    dig_4 = int(cnpj[3]) * 8
    dig_5 = int(cnpj[4]) * 9
    dig_6 = int(cnpj[5]) * 2
    dig_7 = int(cnpj[6]) * 3
    dig_8 = int(cnpj[7]) * 4
    dig_9 = int(cnpj[8]) * 5
    dig_10 = int(cnpj[9]) * 6
    dig_11 = int(cnpj[10]) * 7
    dig_12 = int(cnpj[11]) * 8
    dig_13 = int(cnpj[12]) * 9

    dig_1_ao_13_somados = (
        dig_1
        + dig_2
        + dig_3
        + dig_4
        + dig_5
        + dig_6
        + dig_7
        + dig_8
        + dig_9
        + dig_10
        + dig_11
        + dig_12
        + dig_13
    )

    dig_14 = dig_1_ao_13_somados % 11

    if dig_14 > 9:
        dig_14 = 0

    cnpj_validado = cnpj + str(dig_14)

    cnpj = (
        cnpj_validado[0:2]
        + "."
        + cnpj_validado[2:5]
        + "."
        + cnpj_validado[5:8]
        + "/"
        + cnpj_validado[8:12]
        + "-"
        + cnpj_validado[12:]
    )
    cnpj_normalizado = cnpj.replace(".", "").replace("/", "").replace("-", "")
    if validar:
        if digitos_verificadores == cnpj_validado[12:]:
            return cnpj_normalizado
        else:
            raise Exception("Os dígitos verificadores estão incorretos.")
    else:
        return cnpj_normalizado
